@@ -8,14 +8,17 @@ import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
 
-import org.gcube.portlets.user.gcubewidgets.client.elements.Span;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.client.CKanPublisherService;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.client.CKanPublisherServiceAsync;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.client.events.CloseCreationFormEvent;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.client.events.CloseCreationFormEventHandler;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.client.events.DeleteCustomFieldEvent;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.client.events.DeleteCustomFieldEventHandler;
-import org.gcube.portlets.widgets.ckandatapublisherwidget.client.ui.utils.GcubeDialogExtended;
+import org.gcube.portlets.widgets.ckandatapublisherwidget.client.ui.dataset.resources.AddResourceContainer;
+import org.gcube.portlets.widgets.ckandatapublisherwidget.client.ui.dataset.resources.AddResourceToDataset;
+import org.gcube.portlets.widgets.ckandatapublisherwidget.client.ui.dataset.resources.AddedResourcesSummary;
+import org.gcube.portlets.widgets.ckandatapublisherwidget.client.ui.dataset.resources.ResourcesTable;
+import org.gcube.portlets.widgets.ckandatapublisherwidget.client.ui.dataset.resources.TagsPanel;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.client.ui.utils.InfoIconsLabels;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.shared.DatasetMetadataBean;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.shared.LicensesBean;
@@ -25,7 +28,6 @@ import org.gcube.portlets.widgets.ckandatapublisherwidget.shared.OrganizationBea
 
 import com.github.gwtbootstrap.client.ui.AlertBlock;
 import com.github.gwtbootstrap.client.ui.Button;
-import com.github.gwtbootstrap.client.ui.CheckBox;
 import com.github.gwtbootstrap.client.ui.ControlGroup;
 import com.github.gwtbootstrap.client.ui.Form;
 import com.github.gwtbootstrap.client.ui.Icon;
@@ -36,26 +38,19 @@ import com.github.gwtbootstrap.client.ui.Tab;
 import com.github.gwtbootstrap.client.ui.TabPanel;
 import com.github.gwtbootstrap.client.ui.TextArea;
 import com.github.gwtbootstrap.client.ui.TextBox;
-import com.github.gwtbootstrap.client.ui.base.ListItem;
 import com.github.gwtbootstrap.client.ui.constants.AlertType;
 import com.github.gwtbootstrap.client.ui.constants.ButtonType;
+import com.github.gwtbootstrap.client.ui.constants.ControlGroupType;
 import com.github.gwtbootstrap.client.ui.resources.Bootstrap.Tabs;
 import com.google.gwt.core.client.GWT;
-import com.google.gwt.dom.client.Element;
-import com.google.gwt.dom.client.Style.Cursor;
-import com.google.gwt.dom.client.Style.Display;
 import com.google.gwt.event.dom.client.ChangeEvent;
 import com.google.gwt.event.dom.client.ChangeHandler;
 import com.google.gwt.event.dom.client.ClickEvent;
 import com.google.gwt.event.dom.client.ClickHandler;
-import com.google.gwt.event.dom.client.KeyCodes;
-import com.google.gwt.event.dom.client.KeyDownEvent;
 import com.google.gwt.event.shared.HandlerManager;
 import com.google.gwt.uibinder.client.UiBinder;
 import com.google.gwt.uibinder.client.UiField;
 import com.google.gwt.uibinder.client.UiHandler;
-import com.google.gwt.user.client.DOM;
-import com.google.gwt.user.client.EventListener;
 import com.google.gwt.user.client.Timer;
 import com.google.gwt.user.client.Window;
 import com.google.gwt.user.client.rpc.AsyncCallback;
@@ -63,24 +58,17 @@ import com.google.gwt.user.client.ui.Anchor;
 import com.google.gwt.user.client.ui.Composite;
 import com.google.gwt.user.client.ui.FlowPanel;
 import com.google.gwt.user.client.ui.FocusPanel;
-import com.google.gwt.user.client.ui.HTML;
 import com.google.gwt.user.client.ui.HTMLPanel;
-import com.google.gwt.user.client.ui.IsWidget;
 import com.google.gwt.user.client.ui.SimplePanel;
 import com.google.gwt.user.client.ui.VerticalPanel;
 import com.google.gwt.user.client.ui.Widget;
 
 /**
- * Create metadata form for ckan dataset.
+ * Create metadata form for ckan product.
  * @author Costantino Perciante at ISTI-CNR (costantino.perciante@isti.cnr.it)
  */
 public class CreateDatasetForm extends Composite{
 
-	/**
-	 * Create a remote service proxy to talk to the server-side ckan service.
-	 */
-	private final CKanPublisherServiceAsync ckanServices = GWT.create(CKanPublisherService.class);
-
 	private static EditMetadataFormUiBinder uiBinder = GWT
 			.create(EditMetadataFormUiBinder.class);
 
@@ -91,8 +79,6 @@ public class CreateDatasetForm extends Composite{
 	@UiField HTMLPanel createDatasetMainPanel;
 	@UiField TextBox titleTextBox;
 	@UiField TextArea descriptionTextarea;
-	@UiField TextBox tagsEnterTextBox;
-	@UiField FlowPanel tagsPanel;
 	@UiField ListBox licenseListbox;
 	@UiField ListBox visibilityListbox;
 	@UiField ListBox organizationsListbox;
@@ -115,22 +101,19 @@ public class CreateDatasetForm extends Composite{
 	@UiField Form formSecondStep;
 	@UiField Form formThirdStep;
 	@UiField Button continueButton;
-	@UiField Button goBackButtonFirstOrSecondStep;
+	@UiField Button goBackButtonSecondStep;
 	@UiField Paragraph selectedProfile;
 	@UiField Button goToDatasetButton;
+	@UiField FlowPanel goToDatasetButtonPanel;
 	@UiField Button addResourcesButton;
-	@UiField CheckBox addResourcesCheckBox;
-	@UiField ControlGroup resourcesControlGroup;
 	@UiField SimplePanel workspaceResourcesContainer;
 	@UiField Button continueThirdStep;
 	@UiField Button goBackButtonFirstStep;
 	@UiField Anchor licenseUrlAnchor;
 	@UiField Paragraph unavailableUrl;
+	@UiField TagsPanel tagsPanel;
 
 	// info panels
-	@UiField Icon infoIconTags;
-	@UiField FocusPanel focusPanelTags;
-	@UiField Popover popoverTags;
 	@UiField Icon infoIconLicenses;
 	@UiField FocusPanel focusPanelLicenses;
 	@UiField Popover popoverLicenses;
@@ -158,10 +141,26 @@ public class CreateDatasetForm extends Composite{
 	@UiField Icon infoIconResources;
 	@UiField FocusPanel focusPanelResources;
 	@UiField Popover popoverResources;
+	@UiField Icon infoIconTitle;
+	@UiField FocusPanel focusPanelTitle;
+	@UiField Popover popoverTitle;
 	@UiField ControlGroup metadataProfilesControlGroup;
+	@UiField ControlGroup productTitleGroup;
+	@UiField ControlGroup maintainerControlGroup;
+	@UiField ControlGroup versionControlGroup;
+	@UiField ControlGroup organizationsGroup;
 
-	// error message
+	// Create a remote service proxy to talk to the server-side ckan service.
+	private final CKanPublisherServiceAsync ckanServices = GWT.create(CKanPublisherService.class);
+
+	private static final String REGEX_TITLE_PRODUCT_SUBWORD = "[^a-zA-Z0-9_.-]";
+	private static final String REGEX_MAIL = "\\b[\\w.%-]+@[-.\\w]+\\.[A-Za-z]{2,4}\\b";
+	private static final String NONE_PROFILE = "none";
+
+	// error/info messages
 	protected static final String ERROR_PRODUCT_CREATION = "There was an error while trying to publish your product, sorry.. Retry later";
+	protected static final String PRODUCT_CREATED_OK = "Product correctly created!";
+	private static final String TRYING_TO_CREATE_PRODUCT = "Trying to create product, please wait";
 
 	// tab panel 
 	private TabPanel tabPanel; 
@@ -169,9 +168,6 @@ public class CreateDatasetForm extends Composite{
 	// add resource form
 	private AddResourceToDataset resourceForm;
 
-	// tags list
-	private List<String> tagsList = new ArrayList<String>();
-
 	// the licenses
 	private LicensesBean licenseBean;
 
@@ -204,25 +200,55 @@ public class CreateDatasetForm extends Composite{
 
 	/**
 	 * Invoked in the most general case
-	 * @param owner
+	 * @param eventBus the event bus
 	 */
-	public CreateDatasetForm(String owner, HandlerManager eventBus) {
+	public CreateDatasetForm(HandlerManager eventBus) {
 
-		createDatasetFormBody(false, null, owner, eventBus);
+		createDatasetFormBody(false, null, eventBus);
 
 	}
 
 	/**
-	 * Invoked when the workspace is used.
+	 * Invoked when the workspace is used
 	 * @param idFolderWorkspace
-	 * @param owner
+	 * @param eventBus the event bus
 	 */
-	public CreateDatasetForm(String idFolderWorkspace, String owner, HandlerManager eventBus) {
+	public CreateDatasetForm(String idFolderWorkspace, HandlerManager eventBus) {
 
-		createDatasetFormBody(true, idFolderWorkspace, owner, eventBus);
+		createDatasetFormBody(true, idFolderWorkspace, eventBus);
 
 	}
 
+	/**
+	 * Bind on events
+	 */
+	private void bind() {
+
+		// when a custom field is removed, remove it from the list
+		eventBus.addHandler(DeleteCustomFieldEvent.TYPE, new DeleteCustomFieldEventHandler() {
+
+			@Override
+			public void onRemoveEntry(DeleteCustomFieldEvent event) {
+
+				customFieldEntriesList.remove(event.getRemovedEntry());
+				customFields.remove(event.getRemovedEntry());
+
+			}
+		});
+
+		// on close form
+		eventBus.addHandler(CloseCreationFormEvent.TYPE, new CloseCreationFormEventHandler() {
+
+			@Override
+			public void onClose(CloseCreationFormEvent event) {
+
+				InfoIconsLabels.closeDialogBox(popupOpenedIds);
+
+			}
+
+		});
+
+	}
 
 	/**
 	 * The real constructor
@@ -231,12 +257,10 @@ public class CreateDatasetForm extends Composite{
 	 * @param owner
 	 * @param eventBus
 	 */
-	private void createDatasetFormBody(final boolean isWorkspaceRequest, String idFolderWorkspace, String owner, final HandlerManager eventBus){
+	private void createDatasetFormBody(final boolean isWorkspaceRequest, String idFolderWorkspace, final HandlerManager eventBus){
 
 		initWidget(uiBinder.createAndBindUi(this));
 
-		this.owner = owner;
-
 		// save event bus
 		this.eventBus = eventBus;
 
@@ -257,7 +281,7 @@ public class CreateDatasetForm extends Composite{
 		setAlertBlock("Retrieving information, please wait...", AlertType.INFO, true);
 
 		// get back the licenses and the metadata information
-		ckanServices.getDatasetBean(idFolderWorkspace, owner, new AsyncCallback<DatasetMetadataBean>() {
+		ckanServices.getDatasetBean(idFolderWorkspace, new AsyncCallback<DatasetMetadataBean>() {
 
 			@Override
 			public void onFailure(Throwable caught) {
@@ -305,24 +329,21 @@ public class CreateDatasetForm extends Composite{
 							customFields.add(toAdd);
 
 							// add as tag
-							addTagElement(entry.getKey());
+							tagsPanel.addTagElement(entry.getKey());
 
 						}
 					}
 
 					if(isWorkspaceRequest){
 
-						// enable manage resources checkbox
-						resourcesControlGroup.setVisible(true);
-						addResourcesCheckBox.getElement().getStyle().setDisplay(Display.INLINE_BLOCK);
 						resourcesTable = new ResourcesTable(bean.getResources());
 
 						// if there are not resources, for now just checked it ( and hide so that the step will be skipped) TODO
 						if(bean.getResources() == null || bean.getResources().isEmpty()){
 
-							resourcesControlGroup.setVisible(false);
 							alertNoResources.setType(AlertType.WARNING);
 							alertNoResources.setVisible(true);
+
 						}
 
 					}
@@ -444,7 +465,7 @@ public class CreateDatasetForm extends Composite{
 		}
 
 		// add "none" item again
-		metadataProfilesFormatListbox.addItem("none");
+		metadataProfilesFormatListbox.addItem(NONE_PROFILE);
 
 		// select "none"
 		metadataProfilesFormatListbox.setSelectedIndex(0);
@@ -474,7 +495,7 @@ public class CreateDatasetForm extends Composite{
 					metadataProfilesFormatListbox.setEnabled(true);
 
 					// everything went ok
-					setAlertBlock("", AlertType.ERROR, false);
+					setAlertBlock("", AlertType.DEFAULT, false);
 
 				}else
 					setAlertBlock("Error while retrieving profiles, sorry", AlertType.ERROR, true);
@@ -495,7 +516,7 @@ public class CreateDatasetForm extends Composite{
 	 * Add the items to the listbox and put data into the metadataPanel
 	 * @param receivedBean
 	 */
-	private void prepareMetadataList(DatasetMetadataBean receivedBean) {
+	private void prepareMetadataList(final DatasetMetadataBean receivedBean) {
 
 		List<MetaDataProfileBean> beans = receivedBean.getMetadataList();
 
@@ -512,12 +533,13 @@ public class CreateDatasetForm extends Composite{
 
 						String selectedItem = metadataProfilesFormatListbox.getSelectedItemText();
 
-						if(selectedItem.equals("none")){
+						if(selectedItem.equals(NONE_PROFILE)){
 							// hide the panel
 							metadataFieldsPanel.clear();
 							metadataFieldsPanel.setVisible(false);
+							receivedBean.setChosenProfile(null);
 						}else{
-
+							receivedBean.setChosenProfile(selectedItem);
 							metadataFieldsPanel.clear();
 							addFields(selectedItem);
 						}
@@ -530,9 +552,14 @@ public class CreateDatasetForm extends Composite{
 			metadataProfilesControlGroup.setVisible(false);
 			metadataFieldsPanel.clear();
 			listOfMetadataFields.clear();
+			receivedBean.setChosenProfile(null);
 		}
 	}
 
+	/**
+	 * Add fields of the selected metadata profile to the widget
+	 * @param selectedItem
+	 */
 	protected void addFields(String selectedItem) {
 
 		for(MetaDataProfileBean bean: receivedBean.getMetadataList()){
@@ -546,47 +573,20 @@ public class CreateDatasetForm extends Composite{
 				listOfMetadataFields.clear();
 
 				for (MetadataFieldWrapper field : fields) {
-					MetaDataFieldSkeleton fieldWidget = new MetaDataFieldSkeleton(field, eventBus);
-					metadataFieldsPanel.add(fieldWidget);
-					listOfMetadataFields.add(fieldWidget);
+					MetaDataFieldSkeleton fieldWidget;
+					try {
+						fieldWidget = new MetaDataFieldSkeleton(field, eventBus);
+						metadataFieldsPanel.add(fieldWidget);
+						listOfMetadataFields.add(fieldWidget);
+					} catch (Exception e) {
+						GWT.log("Unable to build such widget", e);
+					}
 				}
-
 				metadataFieldsPanel.setVisible(true);		
 			}
 		}
 	}
 
-	/**
-	 * Bind on events
-	 */
-	private void bind() {
-
-		// when a custom field is removed, remove it from the list
-		eventBus.addHandler(DeleteCustomFieldEvent.TYPE, new DeleteCustomFieldEventHandler() {
-
-			@Override
-			public void onRemoveEntry(DeleteCustomFieldEvent event) {
-
-				customFieldEntriesList.remove(event.getRemovedEntry());
-				customFields.remove(event.getRemovedEntry());
-
-			}
-		});
-
-		// on close form
-		eventBus.addHandler(CloseCreationFormEvent.TYPE, new CloseCreationFormEventHandler() {
-
-			@Override
-			public void onClose(CloseCreationFormEvent event) {
-
-				closeDialogBox();
-
-			}
-
-		});
-
-	}
-
 	@UiHandler("addCustomFieldButton")
 	void addCustomFieldEvent(ClickEvent e){
 
@@ -600,44 +600,78 @@ public class CreateDatasetForm extends Composite{
 	void onContinueButton(ClickEvent e){
 
 		// validate data
-		String errorMsg = validateDataOnContinue();
+		final String errorMsg = validateDataOnContinue();
 
-		if(errorMsg == null){
-			// check what to do 
-			if(isWorkspaceRequest){
-				if(!addResourcesCheckBox.getValue()){
+		if(errorMsg != null){
 
-					// we need to show the page to handle resources one by one from the workspace
-					formFirstStep.setVisible(false);
-					formSecondStep.setVisible(true);
-					formThirdStep.setVisible(false);
-
-					// add the resources to the container panel
-					if(workspaceResourcesContainer.getWidget() == null)
-						workspaceResourcesContainer.add(resourcesTable);
-
-				}else{
-
-					// resources will be added automatically and we can show the  page of the profiles
-					formFirstStep.setVisible(false);
-					formThirdStep.setVisible(true);
-				}
-			}else{
-
-				// this is not a workspace request
-				formFirstStep.setVisible(false);
-				formThirdStep.setVisible(true);
-
-			}
-
-			if(metadataProfilesFormatListbox.getSelectedItemText().equals("none"))
-				selectedProfile.setText("");
-			else
-				selectedProfile.setText("Selected Profile is " + metadataProfilesFormatListbox.getSelectedItemText());
+			alertOnContinue("Please check inserted data [" + errorMsg + "]", AlertType.ERROR);
+			return;
 
 		}else{
-			alertOnContinue("Please check inserted data [" + errorMsg + "]", AlertType.ERROR);
+
+			// better check for title (only if the dataset was not created.. if it is the case, fields are not frozen)
+			if(!titleTextBox.isEnabled())
+				actionsAfterOnContinue();
+			else{
+				alertOnContinue("Checking if a product with such title already exists, please wait...", AlertType.INFO);
+
+				ckanServices.datasetIdAlreadyExists(titleTextBox.getText(), new AsyncCallback<Boolean>() {
+
+					@Override
+					public void onSuccess(Boolean result) {
+
+						if(result){
+
+							alertOnContinue("Sorry but a product with such title already exists, try to change it", AlertType.WARNING);
+
+						}else{
+
+							actionsAfterOnContinue();
+
+						}
+					}
+
+					@Override
+					public void onFailure(Throwable caught) {
+
+						alertOnContinue("Sorry but there was a problem while checking if the inserted data are correct", AlertType.ERROR);
+
+					}
+				});
+			}
 		}
+	}
+
+	/**
+	 * After onContinue ...
+	 */
+	private void actionsAfterOnContinue(){
+
+		// check what to do 
+		if(isWorkspaceRequest){
+
+			// we need to show the page to handle resources one by one from the workspace
+			formFirstStep.setVisible(false);
+			boolean resourcesPresent = receivedBean.getResources() != null && receivedBean.getResources().size() > 0 ? true : false;
+			formSecondStep.setVisible(resourcesPresent);
+			formThirdStep.setVisible(!resourcesPresent);
+
+			// add the resources to the container panel
+			if(workspaceResourcesContainer.getWidget() == null)
+				workspaceResourcesContainer.add(resourcesTable);
+
+		}else{
+
+			// this is not a workspace request
+			formFirstStep.setVisible(false);
+			formThirdStep.setVisible(true);
+
+		}
+
+		if(metadataProfilesFormatListbox.getSelectedItemText().equals(NONE_PROFILE))
+			selectedProfile.setText("");
+		else
+			selectedProfile.setText("Selected Profile is " + metadataProfilesFormatListbox.getSelectedItemText());
 
 	}
 
@@ -653,13 +687,14 @@ public class CreateDatasetForm extends Composite{
 	}
 
 
-	@UiHandler("goBackButtonFirstOrSecondStep")
+	@UiHandler("goBackButtonSecondStep")
 	void onGoBackButton(ClickEvent e){
 
 		// swap forms
-		if(isWorkspaceRequest && !addResourcesCheckBox.getValue()){
-			formFirstStep.setVisible(false);
-			formSecondStep.setVisible(true);
+		if(isWorkspaceRequest){
+			boolean resourcesPresent = receivedBean.getResources() != null && receivedBean.getResources().size() > 0 ? true : false;
+			formFirstStep.setVisible(!resourcesPresent);
+			formSecondStep.setVisible(resourcesPresent);
 		}else{
 			formFirstStep.setVisible(true);
 			formSecondStep.setVisible(false);
@@ -685,20 +720,20 @@ public class CreateDatasetForm extends Composite{
 
 		if(errorMessage != null){
 
-			alertOnCreate("Please check the inserted values and the mandatory fields [" + errorMessage +"]", AlertType.ERROR);
+			alertOnCreate("Please check the inserted values and the mandatory fields [" + errorMessage +"]", AlertType.ERROR, true);
 
 		}
 		else{
 
-			String title = titleTextBox.getValue();
-			String description = descriptionTextarea.getText();
+			String title = titleTextBox.getValue().trim();
+			String description = descriptionTextarea.getText().trim();
 			String selectedLicense = licenseListbox.getSelectedItemText();
 			String visibility = visibilityListbox.getSelectedItemText();
-			long version = Long.valueOf(versionTextbox.getValue());
+			long version = Long.valueOf(versionTextbox.getValue().trim());
 			String author = authorTextbox.getValue();
 			String authorEmail = authorEmailTextbox.getValue();
-			String maintainer = maintainerTextbox.getValue();
-			String maintainerEmail = maintainerEmailTextbox.getValue();
+			String maintainer = maintainerTextbox.getValue().trim();
+			String maintainerEmail = maintainerEmailTextbox.getValue().trim();
 			String chosenOrganizationTitle = organizationsListbox.getSelectedItemText();
 
 			//we need to retrieve the organization's name from this title
@@ -721,7 +756,7 @@ public class CreateDatasetForm extends Composite{
 			receivedBean.setVersion(version);
 			receivedBean.setVisibility(visibility.equals("Public"));
 			receivedBean.setTitle(title);
-			receivedBean.setTags(tagsList);
+			receivedBean.setTags(tagsPanel.getTags());
 			receivedBean.setSelectedOrganization(chosenOrganization);
 
 			Map<String, String> customFieldsMap = new HashMap<String, String>();
@@ -729,7 +764,10 @@ public class CreateDatasetForm extends Composite{
 			// prepare custom fields
 			for (MetaDataFieldSkeleton field : listOfMetadataFields) {
 
-				customFieldsMap.put(field.getFieldName(), field.getFieldCurrentValue());
+				String value = field.getFieldCurrentValue();
+
+				if(!value.isEmpty())
+					customFieldsMap.put(field.getFieldName(), field.getFieldCurrentValue());
 
 			}
 
@@ -737,27 +775,28 @@ public class CreateDatasetForm extends Composite{
 
 				String key = customEntry.getKey();
 				String value = customEntry.getValue();
-				customFieldsMap.put(key, value);
+				if(value != null && !value.isEmpty())
+					customFieldsMap.put(key, value);
 
 			}
 
 			receivedBean.setCustomFields(customFieldsMap);
 
 			// alert 
-			alertOnCreate("Trying to create product, please wait", AlertType.INFO);
+			alertOnCreate(TRYING_TO_CREATE_PRODUCT, AlertType.INFO, false);
 
 			// invoke the create method
 			createButton.setEnabled(false);
-			goBackButtonFirstOrSecondStep.setEnabled(false);
+			goBackButtonSecondStep.setEnabled(false);
 
-			ckanServices.createCKanDataset(receivedBean, isWorkspaceRequest, new AsyncCallback<DatasetMetadataBean>() {
+			ckanServices.createCKanDataset(receivedBean, new AsyncCallback<DatasetMetadataBean>() {
 
 				@Override
 				public void onSuccess(final DatasetMetadataBean createdDatasetBean) {
 
 					if(createdDatasetBean != null){
 
-						alertOnCreate("Product correctly created!", AlertType.SUCCESS);
+						alertOnCreate(PRODUCT_CREATED_OK, AlertType.SUCCESS, false);
 
 						// disable dataset fields
 						disableDatasetFields();
@@ -767,19 +806,20 @@ public class CreateDatasetForm extends Composite{
 
 						// show the go to dataset button
 						final String datasetUrl = createdDatasetBean.getSource();
+						goToDatasetButtonPanel.setVisible(true);
 						goToDatasetButton.setVisible(true);
+						goToDatasetButton.setText(datasetUrl);
 						goToDatasetButton.addClickHandler(new ClickHandler() {
 
 							@Override
 							public void onClick(ClickEvent event) {
-								Window.Location.assign(datasetUrl);
+								Window.open(datasetUrl, "_blank", "");
+								//Window.Location.assign(datasetUrl);
 							}
 						});
 
 						// if we are in the "general case" we need to show a form for adding resources
-						if(isWorkspaceRequest)
-						{
-
+						if(isWorkspaceRequest){
 							// leave to back button, but remove create and add go to dataset
 							createButton.removeFromParent();
 
@@ -817,14 +857,13 @@ public class CreateDatasetForm extends Composite{
 
 									// tab for the added resources
 									Tab addedResources = new Tab();
-									addedResources.add(new AddedResourcesSummary(eventBus, owner));
+									addedResources.add(new AddedResourcesSummary(eventBus));
 									addedResources.setHeading("Added Resource");
 									tabPanel.add(addedResources);
 
 									// add tabs to resources panel
 									tabPanel.selectTab(0);
 
-
 									// form container
 									AddResourceContainer container = new AddResourceContainer(datasetUrl);
 									container.add(tabPanel);
@@ -836,14 +875,14 @@ public class CreateDatasetForm extends Composite{
 						}
 
 					}else{
-						alertOnCreate(ERROR_PRODUCT_CREATION, AlertType.ERROR);
+						alertOnCreate(ERROR_PRODUCT_CREATION, AlertType.ERROR, true);
 					}
 
 				}
 
 				@Override
 				public void onFailure(Throwable caught) {
-					alertOnCreate(ERROR_PRODUCT_CREATION, AlertType.ERROR);
+					alertOnCreate(ERROR_PRODUCT_CREATION, AlertType.ERROR, true);
 				}
 			});
 		}
@@ -855,105 +894,117 @@ public class CreateDatasetForm extends Composite{
 	private void prepareInfoIcons() {
 
 		// tags
-		preparePopupPanelAndPopover(
-				InfoIconsLabels.TAGS_INFO_ID_POPUP,
-				InfoIconsLabels.TAGS_INFO_TEXT,
-				InfoIconsLabels.TAGS_INFO_CAPTION,
-				infoIconTags,
-				popoverTags,
-				focusPanelTags
-				);
+		tagsPanel.prepareIcon(popupOpenedIds);
 
 		// licenses
-		preparePopupPanelAndPopover(
+		InfoIconsLabels.preparePopupPanelAndPopover(
 				InfoIconsLabels.LICENSES_INFO_ID_POPUP,
 				InfoIconsLabels.LICENSES_INFO_TEXT,
 				InfoIconsLabels.LICENSES_INFO_CAPTION,
 				infoIconLicenses,
 				popoverLicenses,
-				focusPanelLicenses
+				focusPanelLicenses,
+				popupOpenedIds
 				);
 
 		// visibility
-		preparePopupPanelAndPopover(
+		InfoIconsLabels.preparePopupPanelAndPopover(
 				InfoIconsLabels.VISIBILITY_INFO_ID_POPUP,
 				InfoIconsLabels.VISIBILITY_INFO_TEXT,
 				InfoIconsLabels.VISIBILITY_INFO_CAPTION,
 				infoIconVisibility,
 				popoverVisibility,
-				focusPanelVisibility
+				focusPanelVisibility,
+				popupOpenedIds
 				);
 
 		// author
-		preparePopupPanelAndPopover(
+		InfoIconsLabels.preparePopupPanelAndPopover(
 				InfoIconsLabels.AUTHOR_INFO_ID_POPUP,
 				InfoIconsLabels.AUTHOR_INFO_TEXT,
 				InfoIconsLabels.AUTHOR_INFO_CAPTION,
 				infoIconAuthor,
 				popoverAuthor,
-				focusPanelAuthor
+				focusPanelAuthor,
+				popupOpenedIds
 				);
 
 		// author's email
-		preparePopupPanelAndPopover(
+		InfoIconsLabels.preparePopupPanelAndPopover(
 				InfoIconsLabels.AUTHOR_EMAIL_INFO_ID_POPUP,
 				InfoIconsLabels.AUTHOR_EMAIL_INFO_TEXT,
 				InfoIconsLabels.AUTHOR_EMAIL_INFO_CAPTION,
 				infoIconAuthorEmail,
 				popoverAuthorEmail,
-				focusPanelAuthorEmail
+				focusPanelAuthorEmail,
+				popupOpenedIds
 				);
 
 		// maintainer
-		preparePopupPanelAndPopover(
+		InfoIconsLabels.preparePopupPanelAndPopover(
 				InfoIconsLabels.MAINTAINER_INFO_ID_POPUP,
 				InfoIconsLabels.MAINTAINER_INFO_TEXT,
 				InfoIconsLabels.MAINTAINER_INFO_CAPTION,
 				infoIconMaintainer,
 				popoverMaintainer,
-				focusPanelMaintainer
+				focusPanelMaintainer,
+				popupOpenedIds
 				);
 
 		// maintainer's email
-		preparePopupPanelAndPopover(
+		InfoIconsLabels.preparePopupPanelAndPopover(
 				InfoIconsLabels.MAINTAINER_EMAIL_INFO_ID_POPUP,
 				InfoIconsLabels.MAINTAINER_EMAIL_INFO_TEXT,
 				InfoIconsLabels.MAINTAINER_EMAIL_INFO_CAPTION,
 				infoIconMaintainerEmail,
 				popoverMaintainerEmail,
-				focusPanelMaintainerEmail
+				focusPanelMaintainerEmail,
+				popupOpenedIds
 				);
 
 		// profiles
-		preparePopupPanelAndPopover(
+		InfoIconsLabels.preparePopupPanelAndPopover(
 				InfoIconsLabels.PROFILES_INFO_ID_POPUP,
 				InfoIconsLabels.PROFILES_INFO_TEXT,
 				InfoIconsLabels.PROFILES_INFO_CAPTION,
 				infoIconProfiles,
 				popoverProfiles,
-				focusPanelProfiles
+				focusPanelProfiles,
+				popupOpenedIds
 				);
 
 		// custom fields
-		preparePopupPanelAndPopover(
+		InfoIconsLabels.preparePopupPanelAndPopover(
 				InfoIconsLabels.CUSTOM_FIELDS_INFO_ID_POPUP,
 				InfoIconsLabels.CUSTOM_FIELDS_INFO_TEXT,
 				InfoIconsLabels.CUSTOM_FIELDS_INFO_CAPTION,
 				infoIconCustomFields,
 				popoverCustomFields,
-				focusPanelCustomFields
+				focusPanelCustomFields,
+				popupOpenedIds
 				);
 
 		// resources field
-		preparePopupPanelAndPopover(
+		InfoIconsLabels.preparePopupPanelAndPopover(
 				InfoIconsLabels.RESOURCES_INFO_ID_POPUP,
 				InfoIconsLabels.RESOURCES_INFO_TEXT,
 				InfoIconsLabels.RESOURCES_INFO_CAPTION,
 				infoIconResources,
 				popoverResources,
-				focusPanelResources
+				focusPanelResources,
+				popupOpenedIds
 				);
 
+		// title
+		InfoIconsLabels.preparePopupPanelAndPopover(
+				InfoIconsLabels.TITLE_INFO_ID_POPUP,
+				InfoIconsLabels.TITLE_INFO_TEXT,
+				InfoIconsLabels.TITLE_INFO_CAPTION,
+				infoIconTitle,
+				popoverTitle,
+				focusPanelTitle,
+				popupOpenedIds
+				);
 	}
 
 	/**
@@ -964,8 +1015,11 @@ public class CreateDatasetForm extends Composite{
 
 		for (MetaDataFieldSkeleton field : listOfMetadataFields) {
 
+			field.removeError();
+
 			String error = field.isFieldValueValid();
 			if(error != null){
+				field.showError();
 				return field.getFieldName() + " is not valid. Suggestion: " + error;
 			}
 		}
@@ -1005,26 +1059,27 @@ public class CreateDatasetForm extends Composite{
 	 * @param text
 	 * @param type
 	 */
-	private void alertOnCreate(String text, AlertType type){
+	private void alertOnCreate(String text, AlertType type, boolean hideAfterAWhile){
 
 		onCreateAlertBlock.setText(text);
 		onCreateAlertBlock.setType(type);
 		onCreateAlertBlock.setVisible(true);
 		createButton.setEnabled(true);
-		goBackButtonFirstOrSecondStep.setEnabled(true);
+		goBackButtonSecondStep.setEnabled(true);
 
-		// hide after some seconds
-		Timer t = new Timer() {
+		if(hideAfterAWhile){
+			// hide after some seconds
+			Timer t = new Timer() {
 
-			@Override
-			public void run() {
+				@Override
+				public void run() {
 
-				onCreateAlertBlock.setVisible(false);
+					onCreateAlertBlock.setVisible(false);
 
-			}
-		};
-
-		t.schedule(10000);
+				}
+			};
+			t.schedule(10000);
+		}
 	}
 
 	/**
@@ -1033,48 +1088,59 @@ public class CreateDatasetForm extends Composite{
 	 */
 	private String validateDataOnContinue() {
 
-		String errorMessage = null;
+		// remove errors
+		productTitleGroup.setType(ControlGroupType.NONE);
+		maintainerControlGroup.setType(ControlGroupType.NONE);
+		versionControlGroup.setType(ControlGroupType.NONE);
+		metadataProfilesControlGroup.setType(ControlGroupType.NONE);
+		organizationsGroup.setType(ControlGroupType.NONE);
 
-		if(titleTextBox.getText().isEmpty()){
-			errorMessage = "Missing title";
-			return errorMessage;
+		String title = titleTextBox.getText().trim();
+		if(title.isEmpty()){
+			productTitleGroup.setType(ControlGroupType.ERROR);
+			return "Missing title";
 		}
 
-		// better check for the title
-		String regexTitleSubWord = "^[a-zA-Z0-9_]+$";
-		String[] splittedTitle = titleTextBox.getText().split(" ");
+		// better check for the title		
+		String[] splittedTitle = title.split(" "); 
 
 		for (String word : splittedTitle) {
-
-			if(!word.matches(regexTitleSubWord))
-				return "Please note that only alphanumeric characters are allowed for the title";
-
+			String replaced = word.replaceAll(REGEX_TITLE_PRODUCT_SUBWORD, "");
+			if(!replaced.equals(word)){
+				productTitleGroup.setType(ControlGroupType.ERROR);
+				return  "Please note not all characters are allowed for the title";
+			}
 		}
 
 		// email reg expression
-		String regexMail = "\\b[\\w.%-]+@[-.\\w]+\\.[A-Za-z]{2,4}\\b";
-		if(!validateByRegExpression(maintainerEmailTextbox.getText(), regexMail)){
-			errorMessage = "Not valid maintainer email";
-			return errorMessage;
+		String maintainerMail = maintainerEmailTextbox.getText();
+		if(!maintainerMail.isEmpty() && !maintainerMail.matches(REGEX_MAIL)){
+			maintainerControlGroup.setType(ControlGroupType.ERROR);
+			return "Not valid maintainer email";
 		}
 
 		// check if version is a number
 		try{
-			Integer.valueOf(versionTextbox.getText());
+			int number = Integer.valueOf(versionTextbox.getText().trim());
+			if(number <= 0)
+				throw new Exception();
 		}catch(Exception e){
-			return errorMessage = "Version must be a natural number";
+			versionControlGroup.setType(ControlGroupType.ERROR);
+			return "Version must be a natural number greater than zero";
 		}
 
 		// check if metadata profile is different from none and its mandatory fields have been fulfilled
 		if(checkSelectedMetaDataProfile()){
-			errorMessage = "You must select a metadata profile different frome none";
+			metadataProfilesControlGroup.setType(ControlGroupType.ERROR);
+			return "You must select a metadata profile different frome none";
 		}
 
 		if(organizationsListbox.getSelectedItemText() == null){
-			errorMessage = "You must select an organization in which you want to publish";
+			organizationsGroup.setType(ControlGroupType.ERROR);
+			return "You must select an organization in which you want to publish";
 		}
 
-		return errorMessage;
+		return null;
 	}
 
 	/**
@@ -1082,17 +1148,7 @@ public class CreateDatasetForm extends Composite{
 	 * @return
 	 */
 	private boolean checkSelectedMetaDataProfile() {
-		return metadataProfilesFormatListbox.getSelectedItemText().equals("none") && (metadataProfilesFormatListbox.getItemCount() != 1);
-	}
-
-	/**
-	 * Validate a text against a regular expression.
-	 * @param textToValidate
-	 * @param regex
-	 * @return
-	 */
-	private boolean validateByRegExpression(String textToValidate, String regex){
-		return textToValidate.matches(regex);
+		return metadataProfilesFormatListbox.getSelectedItemText().equals(NONE_PROFILE) && (metadataProfilesFormatListbox.getItemCount() != 1);
 	}
 
 	@UiHandler("resetButton")
@@ -1104,7 +1160,7 @@ public class CreateDatasetForm extends Composite{
 		versionTextbox.setText("");
 		maintainerTextbox.setText("");
 		maintainerEmailTextbox.setText("");
-		removeTags();
+		tagsPanel.removeTags();
 
 		// delete custom fields
 		for (CustomFieldEntry customField : customFieldEntriesList) {
@@ -1126,22 +1182,14 @@ public class CreateDatasetForm extends Composite{
 		maintainerTextbox.setEnabled(false);
 		maintainerEmailTextbox.setEnabled(false);
 		visibilityListbox.setEnabled(false);
-		tagsEnterTextBox.setEnabled(false);
+		tagsPanel.freeze();
 		licenseListbox.setEnabled(false);
 		organizationsListbox.setEnabled(false);
 		addCustomFieldButton.setEnabled(false);
-		addResourcesCheckBox.setEnabled(false);
+		metadataProfilesFormatListbox.setEnabled(false);
 
-		//	freeze tags
-		for(int i = 0; i < tagsList.size(); i++){
-
-			// get tag widget
-			ListItem tagWidget = (ListItem)tagsPanel.getWidget(i);
-
-			// get the "x" span
-			tagWidget.getWidget(1).removeFromParent();
-
-		}
+		for(CustomFieldEntry ce: customFieldEntriesList)
+			ce.freeze();
 
 		// disable profile fields
 		for (MetaDataFieldSkeleton field : listOfMetadataFields) {
@@ -1169,31 +1217,6 @@ public class CreateDatasetForm extends Composite{
 
 	}
 
-	@UiHandler("tagsEnterTextBox")
-	void onAddTag(KeyDownEvent event){
-
-		if (event.getNativeKeyCode() == KeyCodes.KEY_ENTER) {
-			if (!"".equals(tagsEnterTextBox.getValue().trim())) {
-
-				addTagElement(tagsEnterTextBox);
-
-			}
-		}
-	}
-
-	@UiHandler("addResourcesCheckBox")
-	void onAddResourcesCheckboxPress(ClickEvent e){
-
-		// we need to set all resources to be added if checkbox value is true
-		if(addResourcesCheckBox.getValue()){
-
-			GWT.log("Set all resources to be add");
-			resourcesTable.checkAllResources(true);
-
-		}
-
-	}
-
 	@UiHandler("licenseListbox")
 	void onSelectedLicenseChange(ChangeEvent c){
 
@@ -1228,214 +1251,4 @@ public class CreateDatasetForm extends Composite{
 
 	}
 
-	/**
-	 * Add the tag as an element (inserted by the user)
-	 */
-	private void addTagElement(TextBox itemBox){
-
-		if (itemBox.getValue() != null && !"".equals(itemBox.getValue().trim())) {
-
-			if(tagsList.contains(itemBox.getValue())){
-				itemBox.setValue("");
-				return;
-			}
-
-			// ckan accepts only alphanumeric values
-			String[] subTags = itemBox.getValue().split(" ");
-			if(subTags.length == 1){
-				if(!subTags[0].matches("^[a-zA-Z0-9]*$"))
-					return;
-				if(subTags[0].length() <= 1)
-					return;
-			}else{
-				for (int i = 0; i < subTags.length; i++) {
-					String subTag = subTags[i];
-					if(!subTag.matches("^[a-zA-Z0-9]*$"))
-						return;
-				}
-			}
-
-			final String value = itemBox.getValue();
-			final ListItem displayItem = new ListItem();
-			displayItem.setStyleName("tag-style");
-			Span tagText = new Span(itemBox.getValue());
-
-			Span tagRemove = new Span("x");
-			tagRemove.setTitle("Remove this tag");
-			tagRemove.addClickHandler(new ClickHandler() {
-				public void onClick(ClickEvent clickEvent) {
-					removeTag(displayItem, value);
-				}
-			});
-
-			tagRemove.setStyleName("tag-style-x");
-			displayItem.add(tagText);
-			displayItem.add(tagRemove);
-			itemBox.setValue("");
-			itemBox.setFocus(true);
-			tagsPanel.add(displayItem);
-			tagsList.add(value);
-		}
-	}
-
-	/**
-	 * Add the tag as an element (when publishing from workspace)
-	 */
-	private void addTagElement(final String tag){
-
-		if(tagsList.contains(tag))
-			return;
-
-		// ckan accepts only alphanumeric values
-		String[] subTags = tag.split(" ");
-		if(subTags.length == 1){
-			if(!subTags[0].matches("^[a-zA-Z0-9]*$"))
-				return;
-			if(subTags[0].length() <= 1)
-				return;
-		}else{
-			for (int i = 0; i < subTags.length; i++) {
-				String subTag = subTags[i];
-				if(!subTag.matches("^[a-zA-Z0-9]*$"))
-					return;
-			}
-		}
-
-		final ListItem displayItem = new ListItem();
-		displayItem.setStyleName("tag-style");
-		Span p = new Span(tag);
-
-		Span span = new Span("x");
-		span.setTitle("Remove this tag");
-		span.addClickHandler(new ClickHandler() {
-			public void onClick(ClickEvent clickEvent) {
-				removeTag(displayItem, tag);
-			}
-		});
-
-		span.setStyleName("tag-style-x");
-		displayItem.add(p);
-		displayItem.add(span);
-		tagsPanel.add(displayItem);
-		tagsList.add(tag);
-	}
-
-	/**
-	 * Remove a tag from the list
-	 * @param displayItem
-	 */
-	private void removeTag(ListItem displayItem, String value) {
-
-		tagsList.remove(value);
-		tagsPanel.remove(displayItem);
-
-	}
-
-	/**
-	 * Remove all inserted tags
-	 */
-	private void removeTags(){
-
-		tagsList.clear();
-		tagsPanel.clear();
-
-	}
-
-	/**
-	 * Close any dialog box opened
-	 */
-	private void closeDialogBox() {
-
-		for (String popupid : popupOpenedIds) {
-			GcubeDialogExtended popup = null;
-			try{
-				Element element = DOM.getElementById(popupid);
-				popup = (GcubeDialogExtended) Widget.asWidgetOrNull(getWidget(element));
-				popup.hide();
-			}catch(Exception e){
-				GWT.log("ERROR", e);
-			}	
-		}
-	}
-
-	/**
-	 * Prepare the popover and the gcube popup panel for information.
-	 * @param text
-	 * @param captionText
-	 * @param iconElement
-	 * @param popover
-	 * @param focusPanel
-	 */
-	private void preparePopupPanelAndPopover(
-			final String popupId, 
-			final String text, 
-			final String captionText, 
-			Icon iconElement, 
-			Popover popover, 
-			FocusPanel focusPanel){
-
-		// prepare the popover
-		popover.setText(new HTML("<p style='color:initial'>" + text +"</p>").getHTML());
-		popover.setHeading(new HTML("<b>" + captionText +"</b>").getHTML());
-
-		// set icon cursor
-		iconElement.getElement().getStyle().setCursor(Cursor.HELP);
-
-		// prepare the gcube dialog
-		focusPanel.addClickHandler(new ClickHandler() {
-
-			@Override
-			public void onClick(ClickEvent event) {
-
-				// Retrieve elemnt that should have this id
-				GcubeDialogExtended popup = null;
-				try{
-					Element element = DOM.getElementById(popupId);
-					popup = (GcubeDialogExtended) Widget.asWidgetOrNull(getWidget(element));
-				}catch(Exception e){
-					GWT.log("ERROR", e);
-				}
-
-				// if it doesn't exist, create it
-				if(popup == null){
-
-					popup = new GcubeDialogExtended(captionText, text);
-					popup.getElement().setId(popupId);
-					popup.setModal(false);
-
-					// add its id
-					popupOpenedIds.add(popupId);
-
-				}
-
-				// then center and show
-				popup.center();
-				popup.show();
-
-			}
-		});
-
-	}
-
-	/**
-	 * Check if an element of such type is actually a widget
-	 * @param element
-	 * @return
-	 */
-	public static IsWidget getWidget(Element element) {
-		EventListener listener = DOM
-				.getEventListener(element);
-		// No listener attached to the element, so no widget exist for this
-		// element
-		if (listener == null) {
-			GWT.log("Widget is NULL");
-			return null;
-		}
-		if (listener instanceof Widget) {
-			// GWT uses the widget as event listener
-			GWT.log("Widget is " + listener);
-			return (Widget) listener;
-		}
-		return null;
-	}
 }
