@@ -4,21 +4,21 @@ import java.util.ArrayList;
 import java.util.Calendar;
 import java.util.List;
 import java.util.Map;
+import java.util.concurrent.ConcurrentHashMap;
 
 import javax.servlet.http.HttpSession;
 
 import org.gcube.application.framework.core.session.ASLSession;
 import org.gcube.application.framework.core.session.SessionManager;
-import org.gcube.common.homelibrary.home.HomeLibrary;
-import org.gcube.common.homelibrary.home.workspace.Workspace;
-import org.gcube.common.homelibrary.home.workspace.WorkspaceItem;
-import org.gcube.datacatalogue.ckanutillibrary.CKanUtils;
-import org.gcube.datacatalogue.ckanutillibrary.CkanUtilsFactory;
+import org.gcube.datacatalogue.ckanutillibrary.DataCatalogue;
+import org.gcube.datacatalogue.ckanutillibrary.DataCatalogueFactory;
 import org.gcube.datacatalogue.ckanutillibrary.models.ResourceBean;
-import org.gcube.datacatalogue.ckanutillibrary.utils.ApplicationProfileScopePerUrlReader;
+import org.gcube.datacatalogue.ckanutillibrary.utils.SessionCatalogueAttributes;
+import org.gcube.datacatalogue.ckanutillibrary.utils.UtilMethods;
 import org.gcube.portal.custom.scopemanager.scopehelper.ScopeHelper;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.client.CKanPublisherService;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.server.utils.Utils;
+import org.gcube.portlets.widgets.ckandatapublisherwidget.server.utils.WorkspaceUtils;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.shared.DatasetMetadataBean;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.shared.LicensesBean;
 import org.gcube.portlets.widgets.ckandatapublisherwidget.shared.MetaDataProfileBean;
@@ -48,26 +48,21 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 	public static final String TEST_USER = "test.user";
 	private final static String TEST_SEC_TOKEN = "a1e19695-467f-42b8-966d-bf83dd2382ef";
 
-	// ckan keys for ASL
-	private static final String CKAN_TOKEN_KEY = "ckanToken";
-	private static final String CKAN_LICENSES_KEY = "ckanLicenses"; // licenses
-	private static final String CKAN_ORGANIZATIONS_PUBLISH_KEY = "ckanOrganizationsPublish"; // here he can publish
-	private static final String CKAN_PROFILES_KEY = "ckanProfiles"; // product profiles
-	private final static String CLIENT_PORTLET_URL = "currentClientUrlPortlet"; // set by the ckan portlet in session according
-	 																		  // the user's current url location
+	// map <orgName, scope>
+	private ConcurrentHashMap<String, String> mapOrganizationScope = new ConcurrentHashMap<String, String>();
 
 	/**
 	 * Retrieve an instance of the library for the scope
 	 * @param scope if it is null it is evaluated from the session
 	 * @return
 	 */
-	public CKanUtils getCkanUtilsObj(String scope){
+	public DataCatalogue getCatalogue(String scope){
 
-		CKanUtils instance = null;
+		DataCatalogue instance = null;
 		try{
 			String scopeInWhichDiscover = (scope != null && !scope.isEmpty()) ? scope : getASLSession().getScope();
 			logger.debug("Discovering ckan instance into scope " + scopeInWhichDiscover);
-			instance = CkanUtilsFactory.getFactory().getUtilsPerScope(scopeInWhichDiscover);
+			instance = DataCatalogueFactory.getFactory().getUtilsPerScope(scopeInWhichDiscover);
 		}catch(Exception e){
 			logger.error("Unable to retrieve ckan utils", e);
 		}
@@ -96,7 +91,7 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 	 */
 	public String getDevelopmentUser() {
 		String user = TEST_USER;
-		//user = "costantino.perciante";
+		//		user = "costantino.perciante";
 		return user;
 	}
 
@@ -124,7 +119,7 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 			String scopeInWhichDiscover = (scope != null && !scope.isEmpty()) ? scope : getASLSession().getScope();
 
 			// get the key per scope
-			String keyPerScope = Utils.concatenateSessionKeyScope(CKAN_TOKEN_KEY, scopeInWhichDiscover);
+			String keyPerScope = UtilMethods.concatenateSessionKeyScope(SessionCatalogueAttributes.CKAN_TOKEN_KEY, scopeInWhichDiscover);
 
 			// check if session expired
 			if(username.equals(TEST_USER)){
@@ -140,7 +135,7 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 						logger.debug("Found ckan token into session");
 					}
 					else{
-						token = getCkanUtilsObj(scopeInWhichDiscover).getApiKeyFromUsername(username);
+						token = getCatalogue(scopeInWhichDiscover).getApiKeyFromUsername(username);
 						httpSession.setAttribute(keyPerScope, token);
 						logger.debug("Ckan token has been set for user " + username);
 					}
@@ -168,14 +163,14 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 		HttpSession httpSession = getThreadLocalRequest().getSession();
 
 		// get key 
-		String keyPerScope = Utils.concatenateSessionKeyScope(CKAN_ORGANIZATIONS_PUBLISH_KEY, scope);
+		String keyPerScope = UtilMethods.concatenateSessionKeyScope(SessionCatalogueAttributes.CKAN_ORGANIZATIONS_PUBLISH_KEY, scope);
 
 		if(httpSession.getAttribute(keyPerScope) != null){
 			orgsName = (List<OrganizationBean>)httpSession.getAttribute(keyPerScope);
 			logger.info("List of organizations was into session " + orgsName);
 		}
 		else{
-			orgsName = Utils.getUserOrganizationsListAdmin(scope, username, getASLSession().getGroupName(), this);
+			orgsName = Utils.getUserOrganizationsListAdminEditor(scope, username, getASLSession().getGroupName(), this);
 			httpSession.setAttribute(keyPerScope, orgsName);
 			logger.info("Organizations name for user " + username + " has been saved into session " + orgsName);
 		}
@@ -204,27 +199,26 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 	 * @return
 	 */
 	private String findLicenseIdByLicense(String chosenLicense) {	
-		return getCkanUtilsObj(null).findLicenseIdByLicense(chosenLicense);
+
+		String scope = (String)getThreadLocalRequest().getSession().getAttribute(SessionCatalogueAttributes.SCOPE_CLIENT_PORTLET_URL);
+		return getCatalogue(scope).findLicenseIdByLicenseTitle(chosenLicense);
+
 	}
 
 	@Override
-	// TODO this method will be changed because the list of license can vary according the scope
 	public LicensesBean getLicenses() {
 
-		logger.info("Request for CKAN licenses for scope ");
-
 		ASLSession session = getASLSession();
 		String username = session.getUsername();
-		logger.debug("User in session is " + username);
 
 		// get http session
 		HttpSession httpSession = getThreadLocalRequest().getSession();
-		String currentUrl = (String)httpSession.getAttribute(CLIENT_PORTLET_URL);
+		String scope = (String)httpSession.getAttribute(SessionCatalogueAttributes.SCOPE_CLIENT_PORTLET_URL);
+
+		logger.info("Request for CKAN licenses for scope " + scope);
 
-		String scope = new ApplicationProfileScopePerUrlReader().getScopePerUrl(currentUrl);
-		
 		// get key per scope
-		String keyPerScope = Utils.concatenateSessionKeyScope(CKAN_LICENSES_KEY, scope);
+		String keyPerScope = UtilMethods.concatenateSessionKeyScope(SessionCatalogueAttributes.CKAN_LICENSES_KEY, scope);
 
 		LicensesBean licensesBean = null;
 		if(httpSession.getAttribute(keyPerScope) != null){
@@ -232,12 +226,11 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 			logger.debug("List of licenses was into session");
 		}
 		else{
-			List<CkanLicense> titlesLicenses = getCkanUtilsObj(scope).getLicenses();
+			List<CkanLicense> titlesLicenses = getCatalogue(scope).getLicenses();
 			List<String> titles = new ArrayList<String>();
 			List<String> urls = new ArrayList<String>();
 			for (CkanLicense license : titlesLicenses) {
 				titles.add(license.getTitle());
-
 				String url = (license.getUrl() != null && !license.getUrl().isEmpty()) ? license.getUrl() : "";
 				urls.add(url);
 			}
@@ -250,76 +243,54 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 	}
 
 	@Override
-	public DatasetMetadataBean getDatasetBean(String folderId, String owner){
+	public DatasetMetadataBean getDatasetBean(String folderId){
 
 		DatasetMetadataBean bean = null;
 
-		logger.info("DatasetBean request for " + folderId + " and " + owner);
-
 		ASLSession aslSession = getASLSession();
-		String user = aslSession.getUsername();
-		
+		String userName = aslSession.getUsername();
+
+		logger.info("DatasetBean request for " + folderId + " and " + userName);
+
 		HttpSession httpSession = this.getThreadLocalRequest().getSession();
-		String currentUrl = (String)httpSession.getAttribute(CLIENT_PORTLET_URL);
-		String scope = new ApplicationProfileScopePerUrlReader().getScopePerUrl(currentUrl);
+		String scope = (String)httpSession.getAttribute(SessionCatalogueAttributes.SCOPE_CLIENT_PORTLET_URL);
+
+		logger.debug("Scope recovered from session is " + scope);
+
 		if(isWithinPortal()){
 			try{
 
 				// check if session expired
-				if(user.equals(TEST_USER)){
+				if(userName.equals(TEST_USER)){
 					logger.debug("SESSION EXPIRED");
 					return null;
 				}
 
 				logger.debug("Request dataset metadata bean for folder with id " + folderId
-						+ " whose owner is " + owner);
+						+ " whose owner is " + userName);
 				// get usermanager (liferay)
 				UserManager liferUserManager = new LiferayUserManager();
-				GCubeUser userOwner = liferUserManager.getUserByUsername(owner);
+				GCubeUser userOwner = liferUserManager.getUserByUsername(userName);
 
 				// build bean
 				logger.debug("Building bean");
 				bean = new DatasetMetadataBean();
 
 				bean.setId(folderId);
-				bean.setOwnerIdentifier(owner);
+				bean.setOwnerIdentifier(userName);
 				bean.setVersion(1);
 				bean.setAuthorName(userOwner.getFirstName());
 				bean.setAuthorSurname(userOwner.getLastName());
 				bean.setAuthorEmail(userOwner.getEmail());
 				bean.setMaintainer(userOwner.getFullname());
 				bean.setMaintainerEmail(userOwner.getEmail());
-				bean.setOrganizationList(getUserOrganizationsListAdmin(owner, scope));
+				bean.setOrganizationList(getUserOrganizationsListAdmin(userName, scope));
 
 				// if the request comes from the workspace
 				if(folderId != null && !folderId.isEmpty()){
 
-					Workspace ws = HomeLibrary
-							.getHomeManagerFactory()
-							.getHomeManager()
-							.getHome(owner).getWorkspace();
+					WorkspaceUtils.handleWorkspaceResources(folderId, userName, bean);
 
-					WorkspaceItem retrievedItem = ws.getItem(folderId);
-
-					// set some info
-					String onlyAlphanumeric = retrievedItem.getName().replaceAll("[^A-Za-z0-9]", "");
-					bean.setTitle(onlyAlphanumeric);
-					bean.setDescription(retrievedItem.getDescription());
-
-					// retrieve gcube items of the folder
-					Map<String, String> folderItems = Utils.getGcubeItemProperties(retrievedItem);
-					bean.setCustomFields(folderItems);
-
-					// check the resources within the folder (skip subdirectories)
-					List<String> childrenIds = new ArrayList<String>();
-
-					for (WorkspaceItem file : retrievedItem.getChildren()) {
-						if(!file.isFolder()) // ok, it's a file
-							childrenIds.add(file.getId());
-					}
-
-					List<ResourceBeanWrapper> listOfResources = Utils.getWorkspaceResourcesInformation(childrenIds, ws, user);
-					bean.setResources(listOfResources);
 				}
 
 			}catch(Exception e){
@@ -340,31 +311,13 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 				bean.setAuthorEmail("costantino.perciante@isti.cnr.it");
 				bean.setMaintainer("Costantino Perciante");
 				bean.setMaintainerEmail("costantino.perciante@isti.cnr.it");
-				bean.setOrganizationList(getUserOrganizationsListAdmin(owner, TEST_SCOPE));
-				bean.setOwnerIdentifier(owner);
+				bean.setOrganizationList(getUserOrganizationsListAdmin(userName, TEST_SCOPE));
+				bean.setOwnerIdentifier(userName);
 
 				if(folderId != null && !folderId.isEmpty()){
 
-					Workspace ws = HomeLibrary
-							.getHomeManagerFactory()
-							.getHomeManager()
-							.getHome(owner).getWorkspace();
+					WorkspaceUtils.handleWorkspaceResources(folderId, userName, bean);
 
-					WorkspaceItem retrievedItem = ws.getItem(folderId);
-
-					// retrieve gcube items of the folder
-					Map<String, String> folderItems = Utils.getGcubeItemProperties(retrievedItem);
-					bean.setCustomFields(folderItems);
-
-					// check the resources within the folder (skip subdirectories)
-					List<String> childrenIds = new ArrayList<String>();
-
-					for (WorkspaceItem file : retrievedItem.getChildren()) {
-						if(!file.isFolder()) // ok, it's a file
-							childrenIds.add(file.getId());
-					}
-					List<ResourceBeanWrapper> listOfResources = Utils.getWorkspaceResourcesInformation(childrenIds, ws, user);
-					bean.setResources(listOfResources);
 				}
 
 			}catch(Exception e){
@@ -376,18 +329,17 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 	}
 
 	@Override
-	public DatasetMetadataBean createCKanDataset(DatasetMetadataBean toCreate, boolean isWorkspaceRequest) {
+	public DatasetMetadataBean createCKanDataset(DatasetMetadataBean toCreate) {
 
 		logger.debug("Request for creating a dataset with these information " + toCreate);
 
 		ASLSession aslSession = getASLSession();
-		String user = aslSession.getUsername();
-
+		String userName = aslSession.getUsername();
 
 		try{
 
 			// check if session expired
-			if(user.equals(TEST_USER)){
+			if(userName.equals(TEST_USER)){
 				logger.debug("SESSION EXPIRED");
 				return null;
 			}
@@ -407,34 +359,19 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 			boolean setPublic = toCreate.getVisibility();
 
 			// get the list of resources and convert to ResourceBean
-			List<ResourceBean> resources = new ArrayList<ResourceBean>();
+			List<ResourceBean> resources = null;
 			List<ResourceBeanWrapper> resourcesToAdd = toCreate.getResources();
 
-			if(resourcesToAdd != null && !resourcesToAdd.isEmpty())
-				for (ResourceBeanWrapper resourceBeanWrapper : resourcesToAdd) {
+			// we need to copy such resource in the .catalogue area of the user's ws
+			if(resourcesToAdd != null && !resourcesToAdd.isEmpty()){
 
-					if(resourceBeanWrapper.isToBeAdded()){
-						resources.add(new ResourceBean(
-								resourceBeanWrapper.getUrl(), 
-								resourceBeanWrapper.getName(), 
-								resourceBeanWrapper.getDescription(), 
-								resourceBeanWrapper.getId(),
-								resourceBeanWrapper.getOwner(), 
-								null, 
-								resourceBeanWrapper.getMimeType()));
+				resources = WorkspaceUtils.copyResourcesToUserCatalogueArea(resourcesToAdd, toCreate.getId(), userName, toCreate);
 
-					}
+			}
 
-				}
-
-
-			// TODO this code must be checked against Auth 2.0
-			// The ckan instance in which we will publish is the expressed by the organization name (scope)
-			// This means that if we are in root (e.g. /gcube) and the user choose to publish into vreY
-			// the ckan in which we will publish is the one in scope vreY
-			logger.debug("The user wants to publish in organization with name " + organizationNameOrId + " and current scope name is " + aslSession.getScopeName());
-			String scope = Utils.retrieveScopeFromOrganizationName(organizationNameOrId);
-			CKanUtils utils = getCkanUtilsObj(scope);
+			logger.debug("The user wants to publish in organization with name " + organizationNameOrId);
+			String scope = getScopeFromOrgName(organizationNameOrId);
+			DataCatalogue utils = getCatalogue(scope);
 
 			String datasetId = utils.createCKanDataset(getUserCKanTokenFromSession(scope), title, organizationNameOrId,  author,
 					authorMail,  maintainer,  maintainerMail,  version, description, licenseId,
@@ -447,8 +384,16 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 
 				// retrieve the url
 				String datasetUrl =  utils.getPortletUrl() + "?path=" +  utils.getUrlFromDatasetIdOrName(getUserCKanTokenFromSession(scope), datasetId, true);
-
 				toCreate.setSource(datasetUrl);
+
+				// start a thread that will associate this dataset with the group
+				if(toCreate.getChosenProfile() != null){
+
+					AssociationToGroupThread thread = new AssociationToGroupThread(toCreate.getChosenProfile(), datasetId, userName, utils, organizationNameOrId);
+					thread.start();
+
+				}
+
 				return toCreate;
 
 			}else{
@@ -465,10 +410,14 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 	}
 
 	@Override
-	public ResourceBeanWrapper addResourceToDataset(ResourceBeanWrapper resource, String datasetId, String owner) {
+	public ResourceBeanWrapper addResourceToDataset(ResourceBeanWrapper resource, String datasetId) {
+
+
+		ASLSession session = getASLSession();
+		String username = session.getUsername();
 
 		logger.debug("Incoming request for creating new resource for dataset with id " + datasetId + " and organization name of the dataset is " + resource.getOrganizationNameDatasetParent());
-		logger.debug("Owner is " + owner + " and resource is " + resource);
+		logger.debug("Owner is " + username + " and resource is " + resource);
 
 		if(!isWithinPortal()){
 
@@ -477,9 +426,6 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 
 		}else{
 
-			ASLSession session = getASLSession();
-			String username = session.getUsername();
-
 			if(username.equals(TEST_USER)){
 
 				logger.warn("SESSION EXPIRED! ");
@@ -493,13 +439,13 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 							resource.getName(), 
 							resource.getDescription(), 
 							null, 
-							owner, 
+							username, 
 							datasetId,
 							null);
 
 					// get the scope in which we should discover the ckan instance given the organization name in which the dataset was created
-					String scope = Utils.retrieveScopeFromOrganizationName(resource.getOrganizationNameDatasetParent());
-					String resourceId = getCkanUtilsObj(scope).addResourceToDataset(resourceBean, getUserCKanTokenFromSession(scope));
+					String scope = getScopeFromOrgName(resource.getOrganizationNameDatasetParent());
+					String resourceId = getCatalogue(scope).addResourceToDataset(resourceBean, getUserCKanTokenFromSession(scope));
 
 					if(resourceId != null){
 						logger.debug("Resource  " + resource.getName() + " is now available");
@@ -517,8 +463,7 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 	}
 
 	@Override
-	public boolean deleteResourceFromDataset(ResourceBeanWrapper resource,
-			String owner) {
+	public boolean deleteResourceFromDataset(ResourceBeanWrapper resource) {
 
 		logger.debug("Request for deleting resource " + resource);
 		boolean deleted = false;
@@ -537,8 +482,8 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 			}else{
 				try{
 					// get the scope in which we should discover the ckan instance given the organization name in which the dataset was created
-					String scope = Utils.retrieveScopeFromOrganizationName(resource.getOrganizationNameDatasetParent());
-					deleted = getCkanUtilsObj(scope).
+					String scope = getScopeFromOrgName(resource.getOrganizationNameDatasetParent());
+					deleted = getCatalogue(scope).
 							deleteResourceFromDataset(resource.getId(), getUserCKanTokenFromSession(scope));
 					if(deleted){
 						logger.debug("Resource described by " + resource + " deleted");
@@ -554,18 +499,70 @@ public class CKANPublisherServicesImpl extends RemoteServiceServlet implements C
 
 	@Override
 	public List<MetaDataProfileBean> getProfiles(String orgName) {
-		
+
 		logger.debug("Requested profiles for products into orgName " + orgName);
 
 		List<MetaDataProfileBean> toReturn = new ArrayList<MetaDataProfileBean>();
 		try{
-			String evaluatedScope = Utils.retrieveScopeFromOrganizationName(orgName);
+			String evaluatedScope = getScopeFromOrgName(orgName);
 			logger.debug("Evaluated scope is  " + evaluatedScope);
-			toReturn = Utils.getMetadataProfilesList(evaluatedScope, getThreadLocalRequest().getSession(), getASLSession(), CKAN_PROFILES_KEY);
+			toReturn = Utils.getMetadataProfilesList(evaluatedScope, getThreadLocalRequest().getSession(), getASLSession());
 		}catch(Exception e){
 			logger.error("Failed to retrieve profiles for scope coming from organization name " + orgName, e);
 		}
 
 		return toReturn;
 	}
+
+	@Override
+	public boolean datasetIdAlreadyExists(String title) {
+
+		if(title == null || title.isEmpty())
+			return true; // it's an error somehow
+
+		try{
+
+			String scope = (String)getThreadLocalRequest().getSession().getAttribute(SessionCatalogueAttributes.SCOPE_CLIENT_PORTLET_URL);
+			String idFromTitle = org.gcube.datacatalogue.ckanutillibrary.utils.UtilMethods.fromProductTitleToName(title);
+			return getCatalogue(scope).existProductWithNameOrId(idFromTitle);
+
+		}catch(Exception e){
+			logger.error("Unable to check if such a dataset id already exists", e);
+		}
+
+		return false;
+	}
+
+	/**
+	 * The method tries to retrieve the scope related to the organization using the map first,
+	 * if no match is found, it retrieves such information by using liferay
+	 */
+	private String getScopeFromOrgName(String orgName){
+
+		logger.debug("Request for scope related to orgName " + orgName + "[ map that will be used is " + mapOrganizationScope.toString() + " ]");
+
+		if(orgName == null || orgName.isEmpty())
+			throw new IllegalArgumentException("orgName cannot be empty or null!");
+
+		String toReturn = null;
+
+		if(mapOrganizationScope.containsKey(orgName))
+			toReturn = mapOrganizationScope.get(orgName);
+		else{
+			try{
+
+				String evaluatedScope = Utils.retrieveScopeFromOrganizationName(orgName);
+				mapOrganizationScope.put(orgName, evaluatedScope);
+				toReturn = evaluatedScope;
+
+			}catch(Exception e){
+				logger.error("Failed to retrieve scope from OrgName for organization " + orgName, e);
+			}
+		}
+
+
+		logger.debug("Returning scope " + toReturn);
+		return toReturn;
+
+	}
 }
\ No newline at end of file
